/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "documentcontroller.h"

DocumentController::DocumentController(QObject *parent) : QObject(parent), settings() {
    _document = 0;
    _filePath = QString::Null();

    _page = -1;

    _scaleMode = ScaleFactorMode;
    _displayMode = PagingMode;
    _rotationMode = DoNotRotateMode;

    _scaleFactor = 1.0;
}

DocumentController::~DocumentController() {
    if(_document) { delete _document; }
}

void DocumentController::loadSettings() {
    setScaleMode(static_cast<ScaleModes>(settings.value("documentController/scaleMode", 0).toUInt()));
    setDisplayMode(static_cast<DisplayModes>(settings.value("documentController/displayMode", 0).toUInt()));
    setRotationMode(static_cast<RotationModes>(settings.value("documentController/rotationMode", 0).toUInt()));

    setScaleFactor(settings.value("documentController/scaleFactor", 1.0).toReal());
}

void DocumentController::storeSettings() {
    settings.setValue("documentController/scaleMode", static_cast<uint>(_scaleMode));
    settings.setValue("documentController/displayMode", static_cast<uint>(_displayMode));
    settings.setValue("documentController/rotationMode", static_cast<uint>(_rotationMode));

    settings.setValue("documentController/scaleFactor", _scaleFactor);
}

bool DocumentController::loadFile(const QString &filePath) {
    Poppler::Document *document = Poppler::Document::load(filePath);

    if(document) {
        if(_document) { delete _document; }

        _document = document;


        _filePath = filePath;

        _page = 1;

        emit documentChanged(filePath);
        emit pageChanged(_page);

        document->setRenderHint(Poppler::Document::Antialiasing);
        document->setRenderHint(Poppler::Document::TextAntialiasing);
    }

    return document != 0;
}

bool DocumentController::reload() {
    if(_document) {
        Poppler::Document *document = Poppler::Document::load(_filePath);

        if(document) {
            if(_document) { delete _document; }

            _document = document;


            if(_page > _document->numPages()) {
                _page = 1;
            }

            emit documentChanged(_filePath);
            emit pageChanged(_page);

            document->setRenderHint(Poppler::Document::Antialiasing);
            document->setRenderHint(Poppler::Document::TextAntialiasing);
        }

        return document != 0;
    } else {
        return false;
    }
}

bool DocumentController::saveCopy(const QString &filePath) {
    if(_document) {
        Poppler::PDFConverter *pdfConverter = _document->pdfConverter();

        pdfConverter->setOutputFileName(filePath);
        bool success = pdfConverter->convert();

        delete pdfConverter;

        return success;
    } else {
        return false;
    }
}

int DocumentController::pageCount() const {
    if(_document) {
        return _document->numPages();
    } else {
        return -1;
    }
}

qreal DocumentController::pageWidth() const {
    qreal pageWidth = -1.0;

    if(_document) {
        Poppler::Page *tmpPage=_document->page(_page-1);

        switch(_rotationMode) {
        case DoNotRotateMode:
        case RotateBy180Mode:
            pageWidth = tmpPage->pageSizeF().width();
            break;
        case RotateBy90Mode:
        case RotateBy270Mode:
            pageWidth = tmpPage->pageSizeF().height();
            break;
        }

        delete tmpPage;
    }

    return pageWidth;
}

qreal DocumentController::pageHeight() const {
    qreal pageHeight = -1.0;

    if(_document) {
        Poppler::Page *tmpPage=_document->page(_page-1);

        switch(_rotationMode) {
        case DoNotRotateMode:
        case RotateBy180Mode:
            pageHeight = tmpPage->pageSizeF().height();
            break;
        case RotateBy90Mode:
        case RotateBy270Mode:
            pageHeight = tmpPage->pageSizeF().width();
            break;
        }

        delete tmpPage;
    }

    return pageHeight;
}

QImage DocumentController::renderPage(const qreal &resolutionX, const qreal &resolutionY) const {
    QImage image;

    if(_document) {
        Poppler::Page *tmpPage = _document->page(_page-1);

        switch(_rotationMode) {
        case DoNotRotateMode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate0);
            break;
        case RotateBy90Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate90);
            break;
        case RotateBy180Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate180);
            break;
        case RotateBy270Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate270);
            break;
        }

        delete tmpPage;
    }

    return image;
}

QImage DocumentController::renderPage(const int &page, const qreal &resolutionX, const qreal &resolutionY) const {
    QImage image;

    if(_document) {
        Poppler::Page *tmpPage = _document->page(page-1);

        switch(_rotationMode) {
        case DoNotRotateMode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate0);
            break;
        case RotateBy90Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate90);
            break;
        case RotateBy180Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate180);
            break;
        case RotateBy270Mode:
            image = tmpPage->renderToImage(resolutionX,resolutionY,-1,-1,-1,-1,Poppler::Page::Rotate270);
            break;
        }

        delete tmpPage;
    }

    return image;
}

// slots

void DocumentController::setPage(const int &page) {
    if(_document && page >= 1 && page <= pageCount() && _page != page) {
        if(( _displayMode == DoublePagingMode || _displayMode == DoubleScrollingMode ) && page%2==0) {
            _page = page-1;
        } else {
            _page = page;
        }

        emit pageChanged(_page);
    }
}

void DocumentController::setScaleMode(const DocumentController::ScaleModes &scaleMode) {
    if(_scaleMode != scaleMode) {
        _scaleMode = scaleMode;

        emit scaleModeChanged(scaleMode);
    }
}

void DocumentController::setDisplayMode(const DocumentController::DisplayModes &displayMode) {
    if(_displayMode != displayMode) {
        if(( displayMode == DoublePagingMode || displayMode == DoubleScrollingMode ) && _page%2==0) {
            _page = _page-1;
        }

        _displayMode = displayMode;

        emit displayModeChanged(displayMode);
    }
}

void DocumentController::setRotationMode(const DocumentController::RotationModes &rotationMode) {
    if(_rotationMode != rotationMode) {
        _rotationMode = rotationMode;

        emit rotationModeChanged(rotationMode);
    }
}

void DocumentController::setScaleFactor(const qreal &scaleFactor) {
    if(scaleFactor >= 0.25 && scaleFactor <= 4.0 && _scaleFactor != scaleFactor) {
        _scaleFactor = scaleFactor;

        emit scaleFactorChanged(scaleFactor);
    }
}

void DocumentController::previousPage() {
    switch(_displayMode) {
    case PagingMode:
    case ScrollingMode:
        if(_document && _page > 1) {
            _page--;

            emit pageChanged(_page);
        }
        break;
    case DoublePagingMode:
    case DoubleScrollingMode:
        if(_document && _page > 2) {
            _page -= 2;

            emit pageChanged(_page);
        }
        break;
    }
}

void DocumentController::nextPage() {
    switch(_displayMode) {
    case PagingMode:
    case ScrollingMode:
        if(_document && _page <= pageCount()-1) {
            _page++;

            emit pageChanged(_page);
        }
        break;
    case DoublePagingMode:
    case DoubleScrollingMode:
        if(_document && _page <= pageCount()-2) {
            _page += 2;

            emit pageChanged(_page);
        }
        break;
    }
}

void DocumentController::firstPage() {
    if(_document && _page != 1) {
        _page = 1;

        emit pageChanged(_page);
    }
}

void DocumentController::lastPage() {
    switch(_displayMode) {
    case PagingMode:
    case ScrollingMode:
        if(_document && _page != pageCount()) {
            _page = pageCount();

            emit pageChanged(_page);
        }
        break;
    case DoublePagingMode:
    case DoubleScrollingMode:
        if(_document && pageCount() % 2 == 0) { // even number of pages
            if(_page != pageCount()-1) {
                _page = pageCount()-1;

                emit pageChanged(_page);
            }
        } else if(_document && pageCount() % 2 != 0) { // odd number of pages
            if(_page != pageCount()) {
                _page = pageCount();

                emit pageChanged(_page);
            }

        }
        break;
    }
}
