/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "documentwidget.h"

DocumentWidget::DocumentWidget(QWidget *parent) : QWidget(parent), settings() {
    // controller

    _documentController = new DocumentController();

    connect(_documentController, SIGNAL(documentChanged(QString)), this, SLOT(updateDocument(QString)));

    connect(_documentController, SIGNAL(pageChanged(int)), this, SLOT(updatePage(int)));

    connect(_documentController, SIGNAL(scaleModeChanged(DocumentController::ScaleModes)), this, SLOT(updateScaleMode(DocumentController::ScaleModes)));
    connect(_documentController, SIGNAL(displayModeChanged(DocumentController::DisplayModes)), this, SLOT(updateDisplayMode(DocumentController::DisplayModes)));
    connect(_documentController, SIGNAL(rotationModeChanged(DocumentController::RotationModes)), this, SLOT(updateRotationMode(DocumentController::RotationModes)));

    connect(_documentController, SIGNAL(scaleFactorChanged(qreal)), this, SLOT(updateScaleFactor(qreal)));

    // toolBar

    openDocumentButton = new QPushButton(QIcon::fromTheme("document-open"), QString::Null());
    openDocumentButton->setToolTip(tr("Open document..."));
    connect(openDocumentButton, SIGNAL(clicked()), this, SLOT(openDocument()));

    if(!QIcon::hasThemeIcon("document-open")) { openDocumentButton->setIcon(QIcon(":/document-open.svg")); }

    pageLabel = new QLabel(tr("Page"));
    pageLineEdit = new QLineEdit();
    pageValidator = new QIntValidator();
    pageCountLabel = new QLabel();

    pageLineEdit->setAlignment(Qt::AlignCenter);
    pageLineEdit->setValidator(pageValidator);
    pageLineEdit->setMaximumWidth(50);
    connect(pageLineEdit, SIGNAL(editingFinished()), this, SLOT(setPageByEdit()));

    previousPageButton = new QPushButton(QIcon::fromTheme("go-previous"), QString::Null());
    previousPageButton->setToolTip(tr("Previous page"));
    nextPageButton = new QPushButton(QIcon::fromTheme("go-next"), QString::Null());
    nextPageButton->setToolTip(tr("Next page"));
    firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), QString::Null());
    firstPageButton->setToolTip(tr("First page"));
    lastPageButton = new QPushButton(QIcon::fromTheme("go-last"), QString::Null());
    lastPageButton->setToolTip(tr("Last page"));

    if(!QIcon::hasThemeIcon("go-previous")) { previousPageButton->setIcon(QIcon(":/go-previous.svg")); }
    if(!QIcon::hasThemeIcon("go-next")) { nextPageButton->setIcon(QIcon(":/go-next.svg")); }
    if(!QIcon::hasThemeIcon("go-first")) { firstPageButton->setIcon(QIcon(":/go-first.svg")); }
    if(!QIcon::hasThemeIcon("go-last")) { lastPageButton->setIcon(QIcon(":/go-last.svg")); }

    connect(previousPageButton, SIGNAL(clicked()), _documentController, SLOT(previousPage()));
    connect(nextPageButton, SIGNAL(clicked()), _documentController, SLOT(nextPage()));
    connect(firstPageButton, SIGNAL(clicked()), _documentController, SLOT(firstPage()));
    connect(lastPageButton, SIGNAL(clicked()), _documentController, SLOT(lastPage()));

    /*scaleModeLabel = new QLabel(tr("Scale mode:"));

    scaleModeComboBox = new QComboBox();
    scaleModeComboBox->addItem(tr("Use scale factor"));
    scaleModeComboBox->addItem(tr("Fit to page"));
    scaleModeComboBox->addItem(tr("Fit to page width"));

    connect(scaleModeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setScaleModeByIndex(int)));

    scaleFactorLabel = new QLabel(tr("Scale factor:"));
    scaleFactorSlider = new QSlider(Qt::Horizontal);
    scaleFactorSlider->setRange(25,400);
    scaleFactorSlider->setValue(100);
    scaleFactorSlider->setToolTip(tr("100 %"));

    connect(scaleFactorSlider, SIGNAL(valueChanged(int)), this, SLOT(setScaleFactorByValue(int)));*/

    scalingLabel = new QLabel(tr("Scaling:"));

    scalingComboBox = new QComboBox();
    scalingComboBox->addItem(tr("Fit to page"));
    scalingComboBox->addItem(tr("Fit to page width"));
    scalingComboBox->addItem(tr("25 %")); selectableScaleFactors.append(0.25);
    scalingComboBox->addItem(tr("50 %")); selectableScaleFactors.append(0.50);
    scalingComboBox->addItem(tr("75 %")); selectableScaleFactors.append(0.75);
    scalingComboBox->addItem(tr("100 %")); selectableScaleFactors.append(1.00);
    scalingComboBox->addItem(tr("125 %")); selectableScaleFactors.append(1.25);
    scalingComboBox->addItem(tr("150 %")); selectableScaleFactors.append(1.50);
    scalingComboBox->addItem(tr("175 %")); selectableScaleFactors.append(1.75);
    scalingComboBox->addItem(tr("200 %")); selectableScaleFactors.append(2.00);
    scalingComboBox->addItem(tr("300 %")); selectableScaleFactors.append(3.00);
    scalingComboBox->addItem(tr("400 %")); selectableScaleFactors.append(4.00);
    scalingComboBox->setCurrentIndex( selectableScaleFactors.indexOf(1.0)+2 );

    connect(scalingComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setScalingByIndex(int)));

    toolBarLayout = new QHBoxLayout();

    toolBarLayout->addWidget(openDocumentButton);
    toolBarLayout->addSpacing(15);
    toolBarLayout->addWidget(firstPageButton);
    toolBarLayout->addWidget(previousPageButton);
    toolBarLayout->addSpacing(5);
    toolBarLayout->addWidget(pageLabel);
    toolBarLayout->addWidget(pageLineEdit);
    toolBarLayout->addWidget(pageCountLabel);
    toolBarLayout->addSpacing(5);
    toolBarLayout->addWidget(nextPageButton);
    toolBarLayout->addWidget(lastPageButton);
    /*toolBarLayout->addSpacing(15);
    toolBarLayout->addWidget(scaleModeLabel);
    toolBarLayout->addWidget(scaleModeComboBox);
    toolBarLayout->addSpacing(10);
    toolBarLayout->addWidget(scaleFactorLabel);
    toolBarLayout->addWidget(scaleFactorSlider);*/
    toolBarLayout->addStretch(15);
    toolBarLayout->addWidget(scalingLabel);
    toolBarLayout->addWidget(scalingComboBox);

    toolBar = new QWidget();
    toolBar->setMaximumWidth(750);
    toolBar->setLayout(toolBarLayout);

    // mainView

    mainScene = new QGraphicsScene();
    mainScene->setBackgroundBrush(QBrush(QColor("darkgrey")));

    mainView = new QGraphicsView(mainScene);
    mainView->setInteractive(false);

    connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
    connect(mainView->verticalScrollBar(), SIGNAL(sliderReleased()), this, SLOT(update()));

    mainLayout = new QVBoxLayout();

    mainLayout->addWidget(toolBar);
    mainLayout->addWidget(mainView);

    this->setLayout(mainLayout);

    // imageCache

    imageCacheSize = settings.value("documentWidget/imageCacheSize", 16).toInt();
    imageCache = new QMap<int, QGraphicsPixmapItem*>();

    cachedScaleMode = DocumentController::ScaleFactorMode;
    cachedDisplayMode = DocumentController::PagingMode;
    cachedRotationMode = DocumentController::DoNotRotateMode;

    cachedScaleFactor = 1.0;
}

DocumentWidget::~DocumentWidget() {
    // imageCache

    delete imageCache;

    // mainView

    delete mainScene;
    delete mainView;

    // toolBar

    delete openDocumentButton;

    delete pageLabel;
    delete pageLineEdit;
    delete pageValidator;
    delete pageCountLabel;

    delete previousPageButton;
    delete nextPageButton;
    delete firstPageButton;
    delete lastPageButton;

    /*delete scaleModeLabel;
    delete scaleModeComboBox;

    delete scaleFactorLabel;
    delete scaleFactorSlider;*/

    delete scalingLabel;
    delete scalingComboBox;

    delete toolBar;

    // controller

    delete _documentController;
}

// public slots

void DocumentWidget::openDocument() {
    QString path = settings.value("mainWindow/path", QDir::homePath()).toString();
    QString filePath = QFileDialog::getOpenFileName(this, tr("Open document"), path, tr("Portable document format (*.pdf)"));

    if(filePath != QString::Null()) {
        if(_documentController->loadFile(filePath)) {
            settings.setValue("mainWindow/path", QFileInfo(filePath).path());
        }
    }
}

void DocumentWidget::updateDocument(const QString&) {
    if(_documentController->page() != -1) {
        pageValidator->setRange(1, _documentController->pageCount());
        pageCountLabel->setText(tr("of %1.").arg(_documentController->pageCount()));

        mainView->show();

        prepareScaleMode();
        prepareDisplayMode();

        imageCache->clear();
    } else {
        pageValidator->setRange(1, 1);
        pageCountLabel->setText(QString::Null());

        mainView->hide();
    }

    this->update();
}

void DocumentWidget::updatePage(const int &page) {
    if(page != -1) {
        pageLineEdit->setText(tr("%1").arg(page));

        int page = _documentController->page();
        int pageCount = _documentController->pageCount();

        int visiblePage;

        switch(_documentController->displayMode()) {
        case DocumentController::PagingMode:
        case DocumentController::DoublePagingMode:
            break;
        case DocumentController::ScrollingMode:
            prepareScaleMode();

            visiblePage = qFloor(
                        static_cast<qreal>( pageCount ) *
                        static_cast<qreal>( mainView->verticalScrollBar()->value() ) /
                        static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                        ) + 1;

            if( ( visiblePage <= pageCount && visiblePage != page ) ||
                    ( visiblePage > pageCount && page != pageCount) ) {
                disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

                mainView->ensureVisible(0.0, (page-1) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0, 0);

                connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
            }

            break;
        case DocumentController::DoubleScrollingMode:
            prepareScaleMode();

            if(pageCount%2 == 0) {
                // even page count
                visiblePage = 2 * qFloor(
                            static_cast<qreal>( pageCount / 2 ) *
                            static_cast<qreal>( mainView->verticalScrollBar()->value() ) /
                            static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                            ) + 1;

                if( ( visiblePage <= pageCount && visiblePage != page) ||
                        (visiblePage > pageCount && page != pageCount-1 ) ) {
                    disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

                    mainView->ensureVisible(0.0, ((page-1) / 2) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0, 0);

                    connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
                }
            } else {
                // odd page count
                visiblePage = 2 * qFloor(
                            static_cast<qreal>( (pageCount+1) / 2 ) *
                            static_cast<qreal>( mainView->verticalScrollBar()->value() ) /
                            static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                            ) + 1;

                if( ( visiblePage <= pageCount && visiblePage != page) ||
                        (visiblePage > pageCount && page != pageCount ) ) {
                    disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

                    mainView->ensureVisible(0.0, ((page-1) / 2) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0, 0);

                    connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
                }
            }

            break;
        }
    } else {
        pageLineEdit->setText(QString::Null());
    }

    this->update();
}

void DocumentWidget::updateScaleMode(const DocumentController::ScaleModes &scaleMode) {
    disconnect(scalingComboBox, 0, this, 0);

    switch(scaleMode) {
    case DocumentController::ScaleFactorMode:
        /*scaleModeComboBox->setCurrentIndex(0);
        scaleFactorSlider->setEnabled(true);*/

        scalingComboBox->setCurrentIndex( selectableScaleFactors.indexOf(_documentController->scaleFactor())+2 );

        break;
    case DocumentController::FitToPageMode:
        /*scaleModeComboBox->setCurrentIndex(1);
        scaleFactorSlider->setEnabled(false);*/
        scalingComboBox->setCurrentIndex(0);
        break;
    case DocumentController::FitToPageWidthMode:
        /*scaleModeComboBox->setCurrentIndex(2);
        scaleFactorSlider->setEnabled(false);*/
        scalingComboBox->setCurrentIndex(1);
        break;
    }

    connect(scalingComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setScalingByIndex(int)));

    if(_documentController->page() != -1) {
        prepareScaleMode();
        prepareDisplayMode();
    }

    this->update();
}

void DocumentWidget::updateDisplayMode(const DocumentController::DisplayModes&) {
    if(_documentController->page() != -1) {
        prepareScaleMode();
        prepareDisplayMode();
    }

    this->update();
}

void DocumentWidget::updateRotationMode(const DocumentController::RotationModes &rotationMode) {
    if(_documentController->page() != -1) {
        prepareScaleMode();
        prepareDisplayMode();
    }

    this->update();
}

void DocumentWidget::updateScaleFactor(const qreal &scaleFactor) {
    /*scaleFactorSlider->setValue(qFloor(100.0 * scaleFactor));
    scaleFactorSlider->setToolTip(tr("%1 %").arg(qFloor(100.0 * scaleFactor)));*/

    if(_documentController->scaleMode() == DocumentController::ScaleFactorMode) {
        scalingComboBox->setCurrentIndex( selectableScaleFactors.indexOf(scaleFactor)+2 );
    }

    if(_documentController->page() != -1) {
        prepareScaleMode();
        prepareDisplayMode();
    }

    this->update();
}

// private slots

void DocumentWidget::setPageByEdit() {
    _documentController->setPage(pageLineEdit->text().toInt());
}

void DocumentWidget::setPageByValue(const int &pageValue) {
    if(_documentController->page() != -1) {
        int page = _documentController->page();
        int pageCount = _documentController->pageCount();

        int visiblePage;

        switch(_documentController->displayMode()) {
        case DocumentController::PagingMode:
        case DocumentController::DoublePagingMode:
            break;
        case DocumentController::ScrollingMode:
            visiblePage = qFloor(
                        static_cast<qreal>( pageCount ) *
                        static_cast<qreal>( pageValue ) /
                        static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                        ) + 1;

            if(visiblePage <= pageCount && visiblePage != page) {
                _documentController->setPage(visiblePage);
            } else if(visiblePage > pageCount) {
                _documentController->lastPage();
            }

            break;
        case DocumentController::DoubleScrollingMode:
            if(pageCount % 2 == 0) {
                // even page count
                visiblePage = 2 * qFloor(
                            static_cast<qreal>( pageCount / 2 ) *
                            static_cast<qreal>( pageValue ) /
                            static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                            ) + 1;
            } else {
                // odd page count
                visiblePage = 2 * qFloor(
                            static_cast<qreal>( (pageCount+1) / 2 ) *
                            static_cast<qreal>( pageValue ) /
                            static_cast<qreal>( mainView->verticalScrollBar()->maximum() - mainView->verticalScrollBar()->minimum() )
                            ) + 1;
            }

            if(visiblePage <= pageCount && visiblePage != page) {
                _documentController->setPage(visiblePage);
            } else if(visiblePage > pageCount) {
                _documentController->lastPage();
            }

            break;
        }
    }
}

void DocumentWidget::setScaleModeByIndex(const int &scaleModeIndex) {
    switch(scaleModeIndex) {
    case 0:
        _documentController->setScaleMode(DocumentController::ScaleFactorMode);
        break;
    case 1:
        _documentController->setScaleMode(DocumentController::FitToPageMode);
        break;
    case 2:
        _documentController->setScaleMode(DocumentController::FitToPageWidthMode);
        break;
    }
}

void DocumentWidget::setScaleFactorByValue(const int &scaleFactorValue) {
    _documentController->setScaleFactor(static_cast<qreal>(scaleFactorValue) / 100.0);
}

void DocumentWidget::setScalingByIndex(const int &index) {
    if(index == 0) {
        _documentController->setScaleMode(DocumentController::FitToPageMode);
    } else if(index == 1) {
        _documentController->setScaleMode(DocumentController::FitToPageWidthMode);
    } else {
        _documentController->setScaleMode(DocumentController::ScaleFactorMode);
        _documentController->setScaleFactor(selectableScaleFactors[index-2]);
    }
}

// events

void DocumentWidget::wheelEvent(QWheelEvent *wheelEvent) {
    switch(_documentController->displayMode()) {
    case DocumentController::PagingMode:
        if(wheelEvent->delta() > 0 && mainView->verticalScrollBar()->value() == mainView->verticalScrollBar()->minimum() &&
                _documentController->page() != 1) {
            wheelEvent->accept();

            mainView->verticalScrollBar()->setValue(mainView->verticalScrollBar()->maximum());

            _documentController->previousPage();
        } else if(wheelEvent->delta() < 0 && mainView->verticalScrollBar()->value() == mainView->verticalScrollBar()->maximum() &&
                  _documentController->page() != _documentController->pageCount()) {
            wheelEvent->accept();

            mainView->verticalScrollBar()->setValue(mainView->verticalScrollBar()->minimum());

            _documentController->nextPage();
        } else {
            wheelEvent->ignore();
        }
        break;
    case DocumentController::DoublePagingMode:
        if(wheelEvent->delta() > 0 && mainView->verticalScrollBar()->value() == mainView->verticalScrollBar()->minimum() && _documentController->page() != 1) {
            wheelEvent->accept();

            mainView->verticalScrollBar()->setValue(mainView->verticalScrollBar()->maximum());

            _documentController->previousPage();
        } else if(wheelEvent->delta() < 0 && mainView->verticalScrollBar()->value() == mainView->verticalScrollBar()->maximum() &&
                  ( (_documentController->pageCount()%2==0 && _documentController->page() != _documentController->pageCount()-1) || // even page count
                    (_documentController->pageCount()%2!=0 && _documentController->page() != _documentController->pageCount()) ) ) { // odd page count
            wheelEvent->accept();

            mainView->verticalScrollBar()->setValue(mainView->verticalScrollBar()->minimum());

            _documentController->nextPage();
        } else {
            wheelEvent->ignore();
        }
        break;
    case DocumentController::ScrollingMode:
    case DocumentController::DoubleScrollingMode:
        break;
    }

    this->update();
}

void DocumentWidget::resizeEvent(QResizeEvent*) {
    if(_documentController->page() != -1) {
        prepareScaleMode();
        prepareDisplayMode();
    }

    this->update();
}

void DocumentWidget::paintEvent(QPaintEvent*) {
    if(_documentController->page() != -1) {
        prepareScaleMode();

        int page = _documentController->page();
        int pageCount = _documentController->pageCount();

        switch(_documentController->displayMode()) {
        case DocumentController::PagingMode:
            // current page, left
            renderPage(page, 0.0, 0.0);

            break;
        case DocumentController::DoublePagingMode:
            // current page, left
            renderPage(page, 0.0, 0.0);

            // next page, right
            if(page < pageCount) { renderPage(page+1, 1.05 * pageWidth, 0.0); }

            break;
        case DocumentController::ScrollingMode:
            // previous page, top left
            if(page > 1) { renderPage(page-1, 0.0, (page-2) * 1.05 * pageHeight); }

            // current page, left
            renderPage(page, 0.0, (page-1) * 1.05 * pageHeight);

            // next page, bottom left
            if(page < pageCount) { renderPage(page+1, 0.0, page * 1.05 * pageHeight); }

            // next+1 page, below bottom left
            if(page+1 < pageCount) { renderPage(page+2, 0.0, (page+1) * 1.05 * pageHeight); }

            break;
        case DocumentController::DoubleScrollingMode:
            // previous-1 page, top left
            if(page-1 > 1) { renderPage(page-2, 0.0, ( (page-3) / 2 )* 1.05 * pageHeight); }

            // previous page, top right
            if(page > 1) { renderPage(page-1, 1.05 * pageWidth, ( (page-3) / 2 )* 1.05 * pageHeight); }

            // current page, left
            renderPage(page, 0.0, ( (page-1) / 2 ) * 1.05 * pageHeight);

            // next page, right
            if(page < pageCount) { renderPage(page+1, 1.05 * pageWidth, ( (page-1) / 2 ) * 1.05 * pageHeight); }

            // next+1 page, bottom left
            if(page+1 < pageCount) { renderPage(page+2, 0.0, ( (page+1) / 2 ) * 1.05 * pageHeight); }

            // next+2 page, bottom right
            if(page+2 < pageCount) { renderPage(page+3, 1.05 * pageWidth, ( (page+1) / 2 ) * 1.05 * pageHeight); }

            // next+3 page, below bottom left
            if(page+3 < pageCount) { renderPage(page+4, 0.0, ( (page+3) / 2 ) * 1.05 * pageHeight); }

            // next+4 page, below bottom right
            if(page+4 < pageCount) { renderPage(page+5, 1.05 * pageWidth, ( (page+3) / 2 ) * 1.05 * pageHeight); }

            break;
        }
    } else {
        mainScene->clear();
    }
}

// internal methods

void DocumentWidget::prepareScaleMode() {
    pageWidth = _documentController->pageWidth() * this->physicalDpiX() / 72.0;
    pageHeight = _documentController->pageHeight() * this->physicalDpiY() / 72.0;

    switch(_documentController->displayMode()) {
    case DocumentController::PagingMode:
    case DocumentController::ScrollingMode:
        fitToPageWidth = 0.95 * static_cast<qreal>(mainView->width()) / pageWidth;
        break;
    case DocumentController::DoublePagingMode:
    case DocumentController::DoubleScrollingMode:
        fitToPageWidth = 0.95 * static_cast<qreal>(mainView->width()) / ( 2.05 * pageWidth );
        break;
    }

    fitToPageHeight = 0.95 * static_cast<qreal>(mainView->height()) / pageHeight;

    fitToPage = qMin(fitToPageWidth, fitToPageHeight);

    switch(_documentController->scaleMode()) {
    case DocumentController::ScaleFactorMode:
        pageWidth *= _documentController->scaleFactor();
        pageHeight *= _documentController->scaleFactor();
        break;
    case DocumentController::FitToPageMode:
        pageWidth *= fitToPage;
        pageHeight *= fitToPage;
        break;
    case DocumentController::FitToPageWidthMode:
        pageWidth *= fitToPageWidth;
        pageHeight *= fitToPageWidth;
        break;
    }
}

void DocumentWidget::prepareDisplayMode() {
    switch(_documentController->displayMode()) {
    case DocumentController::PagingMode:
        mainScene->setSceneRect(0.0, 0.0, pageWidth, pageHeight);
        break;
    case DocumentController::DoublePagingMode:
        mainScene->setSceneRect(0.0, 0.0, 2.05 * pageWidth, pageHeight);
        break;
    case DocumentController::ScrollingMode:
        mainScene->setSceneRect(0.0, 0.0, pageWidth, _documentController->pageCount() * 1.05 * pageHeight);

        disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

        mainView->ensureVisible(0.0, (_documentController->page()-1) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0,0);

        connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
        break;
    case DocumentController::DoubleScrollingMode:
        if(_documentController->pageCount() % 2 == 0) {
            // even page count
            mainScene->setSceneRect(0.0, 0.0, 2.05 * pageWidth, ( _documentController->pageCount() / 2 ) * 1.05 * pageHeight);

            disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

            mainView->ensureVisible(0.0, ( (_documentController->page()-1) / 2 ) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0,0);

            connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
        } else {
            // odd page count
            mainScene->setSceneRect(0.0, 0.0, 2.05 * pageWidth, ( (_documentController->pageCount()+1) / 2 ) * 1.05 * pageHeight);

            disconnect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));

            mainView->ensureVisible(0.0, ( (_documentController->page()-1) / 2 ) * 1.05 * pageHeight, mainView->width(), mainView->height(), 0,0);

            connect(mainView->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setPageByValue(int)));
        }
        break;
    }
}

// image cache

void DocumentWidget::renderPage(const int &page, qreal x, qreal y) {
    if(mainView->verticalScrollBar()->isSliderDown()) {
        QGraphicsItem *rect = mainScene->addRect(x, y, pageWidth, pageHeight, QPen(Qt::NoPen), QBrush(QColor("white")));

        rect->setZValue(-1.0);
        return;
    }

    DocumentController::ScaleModes scaleMode = _documentController->scaleMode();
    DocumentController::DisplayModes displayMode = _documentController->displayMode();
    DocumentController::RotationModes rotationMode = _documentController->rotationMode();

    qreal scaleFactor = 1.0;

    switch(_documentController->scaleMode()) {
    case DocumentController::ScaleFactorMode:
        scaleFactor = _documentController->scaleFactor();
        break;
    case DocumentController::FitToPageMode:
        scaleFactor = fitToPage;
        break;
    case DocumentController::FitToPageWidthMode:
        scaleFactor = fitToPageWidth;
        break;
    }

    if(cachedScaleMode == scaleMode && cachedDisplayMode == displayMode && cachedRotationMode == rotationMode && cachedScaleFactor == scaleFactor) {
        if(imageCache->contains(page)) {
            // cache hit
            imageCache->value(page)->setPos(x,y);

            if(_documentController->displayMode() == DocumentController::PagingMode || _documentController->displayMode() == DocumentController::DoublePagingMode) {
                imageCache->value(page)->setVisible(true);

                foreach(QGraphicsItem *item, imageCache->value(page)->collidingItems()) { item->setVisible(false); }
            }
        } else {
            // cache miss
            QImage image = _documentController->renderPage(page, scaleFactor * this->physicalDpiX(), scaleFactor * this->physicalDpiY());
            QGraphicsPixmapItem *pixmap = mainScene->addPixmap(QPixmap::fromImage(image));

            pixmap->setPos(x,y); pixmap->setZValue(1.0);
            imageCache->insert(page, pixmap);

            if(_documentController->displayMode() == DocumentController::PagingMode || _documentController->displayMode() == DocumentController::DoublePagingMode) {
                foreach(QGraphicsItem *item, pixmap->collidingItems()) { item->setVisible(false); }
            }

            if(imageCache->size() > imageCacheSize) {
                // cache shrink
                if(imageCache->upperBound(page) == imageCache->end()) {
                    // forward miss
                    QMap<int, QGraphicsPixmapItem*>::const_iterator i = imageCache->begin();
                    int j = 0;

                    while(i != imageCache->end() && j < imageCacheSize/2) {
                        mainScene->removeItem(i.value());
                        delete i.value();

                        imageCache->remove(i.key());

                        ++i;
                        ++j;
                    }
                } else {
                    // backward miss
                    QMap<int, QGraphicsPixmapItem*>::const_iterator i = --imageCache->end();
                    int j = 0;

                    do {
                        mainScene->removeItem(i.value());
                        delete i.value();

                        imageCache->remove(i.key());

                        --i;
                        ++j;
                    } while(i != imageCache->begin() && j <= imageCacheSize/2);
                }
            }
        }
    } else {
        // cache drop
        cachedScaleMode = scaleMode;
        cachedDisplayMode = displayMode;
        cachedRotationMode = rotationMode;

        cachedScaleFactor = scaleFactor;

        mainScene->clear();
        imageCache->clear();

        QImage image = _documentController->renderPage(page, scaleFactor * this->physicalDpiX(), scaleFactor * this->physicalDpiY());
        QGraphicsPixmapItem *pixmap = mainScene->addPixmap(QPixmap::fromImage(image));

        pixmap->setPos(x,y); pixmap->setZValue(1.0);
        imageCache->insert(page, pixmap);
    }
}
