/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), settings() {
    // menuBar

    // fileMenu

    openDocumentAction = new QAction(tr("&Open document..."), this);
    openDocumentAction->setShortcut(QKeySequence::Open);
    openDocumentAction->setIcon(QIcon::fromTheme("document-open"));
    openDocumentAction->setIconVisibleInMenu(true);
    connect(openDocumentAction, SIGNAL(triggered()), this, SLOT(openDocument()));

    addTabAction = new QAction(tr("&Add tab..."), this);
    addTabAction->setShortcut(QKeySequence::AddTab);
    addTabAction->setIcon(QIcon::fromTheme("tab-new"));
    addTabAction->setIconVisibleInMenu(true);
    connect(addTabAction, SIGNAL(triggered()), this, SLOT(addTab()));

    previousTabAction = new QAction(tr("Previous tab"), this);
    previousTabAction->setShortcut(QKeySequence::PreviousChild);
    connect(previousTabAction, SIGNAL(triggered()), this, SLOT(previousTab()));
    nextTabAction = new QAction(tr("&Next tab"), this);
    nextTabAction->setShortcut(QKeySequence::NextChild);
    connect(nextTabAction, SIGNAL(triggered()), this, SLOT(nextTab()));

    closeTabAction = new QAction(tr("&Close tab"), this);
    closeTabAction->setShortcut(QKeySequence::Close);
    closeTabAction->setIcon(QIcon::fromTheme("window-close"));
    closeTabAction->setIconVisibleInMenu(true);
    connect(closeTabAction, SIGNAL(triggered()), this, SLOT(closeTab()));

    reloadAction = new QAction(tr("&Reload"), this);
    reloadAction->setShortcut(QKeySequence::Refresh);
    reloadAction->setIcon(QIcon::fromTheme("view-refresh"));
    reloadAction->setIconVisibleInMenu(true);
    connect(reloadAction, SIGNAL(triggered()), this, SLOT(reload()));

    printAction = new QAction(tr("&Print..."), this);
    printAction->setShortcut(QKeySequence::Print);
    printAction->setIcon(QIcon::fromTheme("document-print"));
    printAction->setIconVisibleInMenu(true);
    connect(printAction, SIGNAL(triggered()), this, SLOT(print()));

    saveCopyAction = new QAction(tr("&Save copy..."), this);
    saveCopyAction->setShortcut(QKeySequence::Save);
    saveCopyAction->setIcon(QIcon::fromTheme("document-save"));
    saveCopyAction->setIconVisibleInMenu(true);
    connect(saveCopyAction, SIGNAL(triggered()), this, SLOT(saveCopy()));

    quitAction = new QAction(tr("&Exit"), this);
    quitAction->setShortcut(QKeySequence::Quit);
    quitAction->setIcon(QIcon::fromTheme("application-exit"));
    quitAction->setIconVisibleInMenu(true);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));

    previousTabAction->setEnabled(false);
    nextTabAction->setEnabled(false);
    closeTabAction->setEnabled(false);

    reloadAction->setEnabled(false);
    printAction->setEnabled(false);
    saveCopyAction->setEnabled(false);

    QMenu *fileMenu = this->menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openDocumentAction);
    fileMenu->addAction(addTabAction);
    fileMenu->addAction(previousTabAction);
    fileMenu->addAction(nextTabAction);
    fileMenu->addAction(closeTabAction);
    fileMenu->addSeparator();
    fileMenu->addAction(reloadAction);
    fileMenu->addAction(printAction);
    fileMenu->addAction(saveCopyAction);
    fileMenu->addSeparator();
    fileMenu->addAction(quitAction);

    // viewMenu

    previousPageAction = new QAction(tr("&Previous page"), this);
    previousPageAction->setShortcut(QKeySequence::MoveToPreviousPage);
    previousPageAction->setIcon(QIcon::fromTheme("go-previous"));
    previousPageAction->setIconVisibleInMenu(true);

    nextPageAction = new QAction(tr("&Next page"), this);
    nextPageAction->setShortcut(QKeySequence::MoveToNextPage);
    nextPageAction->setIcon(QIcon::fromTheme("go-next"));
    nextPageAction->setIconVisibleInMenu(true);

    firstPageAction = new QAction(tr("&First page"), this);
    firstPageAction->setShortcut(QKeySequence::MoveToStartOfDocument);
    firstPageAction->setIcon(QIcon::fromTheme("go-first"));
    firstPageAction->setIconVisibleInMenu(true);

    lastPageAction = new QAction(tr("&Last page"), this);
    lastPageAction->setShortcut(QKeySequence::MoveToEndOfDocument);
    lastPageAction->setIcon(QIcon::fromTheme("go-last"));
    lastPageAction->setIconVisibleInMenu(true);

    previousPageAction->setEnabled(false);
    nextPageAction->setEnabled(false);
    firstPageAction->setEnabled(false);
    lastPageAction->setEnabled(false);

    // scaleModeActionGroup

    scaleFactorModeAction = new QAction(tr("Use scale factor"), this);
    scaleFactorModeAction->setCheckable(true);

    fitToPageModeAction = new QAction(tr("Fit to page"), this);
    fitToPageModeAction->setCheckable(true);

    fitToPageWidthModeAction = new QAction(tr("Fit to page width"), this);
    fitToPageWidthModeAction->setCheckable(true);

    scaleModeActionGroup = new QActionGroup(this);
    scaleModeActionGroup->addAction(scaleFactorModeAction);
    scaleModeActionGroup->addAction(fitToPageModeAction);
    scaleModeActionGroup->addAction(fitToPageWidthModeAction);

    scaleFactorModeAction->setChecked(true);
    scaleModeActionGroup->setEnabled(false);

    connect(scaleModeActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(setScaleModeByAction(QAction*)));

    // displayModeActionGroup

    pagingModeAction = new QAction(tr("Paging display"), this);
    pagingModeAction->setCheckable(true);

    scrollingModeAction = new QAction(tr("Scrolling display"), this);
    scrollingModeAction->setCheckable(true);

    doublePagingModeAction = new QAction(tr("Double paging display"), this);
    doublePagingModeAction->setCheckable(true);

    doubleScrollingModeAction = new QAction(tr("Double scrolling display"), this);
    doubleScrollingModeAction->setCheckable(true);

    displayModeActionGroup = new QActionGroup(this);
    displayModeActionGroup->addAction(pagingModeAction);
    displayModeActionGroup->addAction(scrollingModeAction);
    displayModeActionGroup->addAction(doublePagingModeAction);
    displayModeActionGroup->addAction(doubleScrollingModeAction);

    pagingModeAction->setChecked(true);
    displayModeActionGroup->setEnabled(false);

    connect(displayModeActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(setDisplayModeByAction(QAction*)));

    // rotationModeActionGroup

    doNotRotateAction = new QAction(tr("Do not rotate"), this);
    doNotRotateAction->setCheckable(true);

    rotateBy90Action = new QAction(tr("Rotate by 90 degrees"), this);
    rotateBy90Action->setCheckable(true);

    rotateBy180Action = new QAction(tr("Rotate by 180 degrees"), this);
    rotateBy180Action->setCheckable(true);

    rotateBy270Action = new QAction(tr("Rotate by 270 degrees"), this);
    rotateBy270Action->setCheckable(true);

    rotationModeActionGroup = new QActionGroup(this);
    rotationModeActionGroup->addAction(doNotRotateAction);
    rotationModeActionGroup->addAction(rotateBy90Action);
    rotationModeActionGroup->addAction(rotateBy180Action);
    rotationModeActionGroup->addAction(rotateBy270Action);

    doNotRotateAction->setChecked(true);
    rotationModeActionGroup->setEnabled(false);

    connect(rotationModeActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(setRotationModeByAction(QAction*)));

    fullscreenAction = new QAction(tr("Full&screen"), this);
    fullscreenAction->setCheckable(true);
    fullscreenAction->setChecked(false);
    fullscreenAction->setShortcut(QKeySequence(Qt::Key_F11));
    fullscreenAction->setIcon(QIcon::fromTheme("view-fullscreen"));
    fullscreenAction->setIconVisibleInMenu(true);
    connect(fullscreenAction, SIGNAL(changed()), this, SLOT(fullscreen()));

    QMenu *viewMenu = this->menuBar()->addMenu(tr("&View"));
    viewMenu->addAction(previousPageAction);
    viewMenu->addAction(nextPageAction);
    viewMenu->addAction(firstPageAction);
    viewMenu->addAction(lastPageAction);
    viewMenu->addSeparator();
    viewMenu->addAction(scaleFactorModeAction);
    viewMenu->addAction(fitToPageModeAction);
    viewMenu->addAction(fitToPageWidthModeAction);
    viewMenu->addSeparator();
    viewMenu->addAction(pagingModeAction);
    viewMenu->addAction(scrollingModeAction);
    viewMenu->addAction(doublePagingModeAction);
    viewMenu->addAction(doubleScrollingModeAction);
    viewMenu->addSeparator();
    viewMenu->addAction(doNotRotateAction);
    viewMenu->addAction(rotateBy90Action);
    viewMenu->addAction(rotateBy180Action);
    viewMenu->addAction(rotateBy270Action);
    viewMenu->addSeparator();
    viewMenu->addAction(fullscreenAction);

    // helpMenu

    aboutAction = new QAction(tr("&About"), this);
    aboutAction->setIcon(QIcon::fromTheme("help-about"));
    aboutAction->setIconVisibleInMenu(true);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));

    QMenu *helpMenu = this->menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAction);

    // tabWidget

    tabWidget = new QTabWidget();
    tabWidget->setMovable(true);
    tabWidget->setTabsClosable(true);
    tabWidget->setDocumentMode(true);
    tabWidget->setElideMode(Qt::ElideRight);

    connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(updateCurrentTab(int)));
    connect(tabWidget, SIGNAL(tabCloseRequested(int)), this, SLOT(closeTabByIndex(int)));

    this->setCentralWidget(tabWidget);

    // geometry

    this->setMinimumSize(500,700);
    this->restoreGeometry(settings.value("mainWindow/geometry").toByteArray());

    geometryBeforeFullscreen = this->saveGeometry();

    // drag and drop

    this->setAcceptDrops(true);

    // command line arguments

    QStringList arguments = QApplication::arguments();

    arguments.removeFirst();

    foreach(QString argument, arguments) {
        if(QFileInfo(argument).isFile()) {
            DocumentWidget *documentWidget = new DocumentWidget();

            if(documentWidget->documentController()->loadFile(argument)) {
                int index = tabWidget->addTab(documentWidget, QString::Null());

                tabWidget->setTabText(index, QFileInfo(argument).baseName());
                tabWidget->setTabToolTip(index, QFileInfo(argument).baseName());

                tabWidget->setCurrentIndex(index);

                connect(documentWidget->documentController(), SIGNAL(documentChanged(QString)), this, SLOT(updateDocument(QString)));

                connect(documentWidget->documentController(), SIGNAL(scaleModeChanged(DocumentController::ScaleModes)), this, SLOT(updateScaleMode(DocumentController::ScaleModes)));
                connect(documentWidget->documentController(), SIGNAL(displayModeChanged(DocumentController::DisplayModes)), this, SLOT(updateDisplayMode(DocumentController::DisplayModes)));
                connect(documentWidget->documentController(), SIGNAL(rotationModeChanged(DocumentController::RotationModes)), this, SLOT(updateRotationMode(DocumentController::RotationModes)));

                documentWidget->documentController()->loadSettings();
            } else {
                delete documentWidget;
            }
        }
    }

    // printing

    printer = 0;
    printThread = 0;
}

MainWindow::~MainWindow() {
    delete openDocumentAction;
    delete addTabAction;
    delete previousTabAction;
    delete nextTabAction;
    delete closeTabAction;

    delete reloadAction;
    delete saveCopyAction;

    delete quitAction;

    delete previousPageAction;
    delete nextPageAction;
    delete firstPageAction;
    delete lastPageAction;

    delete scaleFactorModeAction;
    delete fitToPageModeAction;
    delete fitToPageWidthModeAction;
    delete scaleModeActionGroup;

    delete pagingModeAction;
    delete scrollingModeAction;
    delete doublePagingModeAction;
    delete doubleScrollingModeAction;
    delete displayModeActionGroup;

    delete doNotRotateAction;
    delete rotateBy90Action;
    delete rotateBy180Action;
    delete rotateBy270Action;
    delete rotationModeActionGroup;

    delete fullscreenAction;

    delete aboutAction;
}

// public slots

void MainWindow::openDocument() {
    if(tabWidget->currentIndex() != -1) {
        QString path = settings.value("mainWindow/path", QDir::homePath()).toString();
        QString filePath = QFileDialog::getOpenFileName(this, tr("Open document"), path, tr("Portable document format (*.pdf)"));

        if(filePath != QString::Null()) {
            DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

            if(documentWidget->documentController()->loadFile(filePath)) {
                settings.setValue("mainWindow/path", QFileInfo(filePath).path());
            }
        }
    } else {
        this->addTab();
    }
}

void MainWindow::addTab() {
    QString path = settings.value("mainWindow/path", QDir::homePath()).toString();
    QStringList filePaths = QFileDialog::getOpenFileNames(this, tr("Open document"), path, tr("Portable document format (*.pdf)"));

    foreach(QString filePath, filePaths) {
        DocumentWidget *documentWidget = new DocumentWidget();

        if(documentWidget->documentController()->loadFile(filePath)) {
            int index = tabWidget->addTab(documentWidget, QString::Null());

            tabWidget->setTabText(index, QFileInfo(filePath).baseName());
            tabWidget->setTabToolTip(index, QFileInfo(filePath).baseName());

            tabWidget->setCurrentIndex(index);

            connect(documentWidget->documentController(), SIGNAL(documentChanged(QString)), this, SLOT(updateDocument(QString)));

            connect(documentWidget->documentController(), SIGNAL(scaleModeChanged(DocumentController::ScaleModes)), this, SLOT(updateScaleMode(DocumentController::ScaleModes)));
            connect(documentWidget->documentController(), SIGNAL(displayModeChanged(DocumentController::DisplayModes)), this, SLOT(updateDisplayMode(DocumentController::DisplayModes)));
            connect(documentWidget->documentController(), SIGNAL(rotationModeChanged(DocumentController::RotationModes)), this, SLOT(updateRotationMode(DocumentController::RotationModes)));

            documentWidget->documentController()->loadSettings();

            settings.setValue("mainWindow/path", QFileInfo(filePath).path());
        } else {
            delete documentWidget;
        }
    }
}

void MainWindow::previousTab() {
    if(tabWidget->currentIndex() != -1) {
        if(tabWidget->currentIndex() > 0) {
            tabWidget->setCurrentIndex(tabWidget->currentIndex()-1);
        } else {
            tabWidget->setCurrentIndex(tabWidget->count()-1);
        }
    }
}

void MainWindow::nextTab() {
    if(tabWidget->currentIndex() != -1) {
        if(tabWidget->currentIndex() < tabWidget->count()-1) {
            tabWidget->setCurrentIndex(tabWidget->currentIndex()+1);
        } else {
            tabWidget->setCurrentIndex(0);
        }
    }
}

void MainWindow::closeTab() {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        disconnect(previousPageAction, 0);
        disconnect(nextPageAction, 0);
        disconnect(firstPageAction, 0);
        disconnect(lastPageAction, 0);

        disconnect(documentWidget->documentController(), 0);

        tabWidget->removeTab(tabWidget->currentIndex());

        documentWidget->documentController()->storeSettings();

        delete documentWidget;
    }
}

void MainWindow::reload() {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        if(!documentWidget->documentController()->reload()) {
            QMessageBox::warning(this, tr("Warning"), tr("Could not reload document."));
        }
    }
}

void MainWindow::print() {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        printer = new QPrinter();

        QPrintDialog printDialog(printer, this);
        printDialog.setMinMax(1, documentWidget->documentController()->pageCount());

        if(printDialog.exec() == QDialog::Accepted) {
            int fromPage = 1;
            int toPage = documentWidget->documentController()->pageCount();

            if(printDialog.fromPage() != 0 && printDialog.toPage() != 0) {
                fromPage = printDialog.fromPage();
                toPage = printDialog.toPage();
            }

            printThread = new PrintThread(documentWidget->documentController(), printer, fromPage, toPage, this);

            connect(printThread, SIGNAL(beginPrinting()), this, SLOT(beginPrinting()));
            connect(printThread, SIGNAL(continuePrinting(QString)), this, SLOT(continuePrinting(QString)));
            connect(printThread, SIGNAL(endPrinting()), this, SLOT(endPrinting()));

            printThread->start();
        }
    }
}

void MainWindow::beginPrinting() {
    this->setEnabled(false);
}

void MainWindow::continuePrinting(const QString &progress) {
    this->statusBar()->showMessage(progress);
}

void MainWindow::endPrinting() {
    this->setEnabled(true);

    this->statusBar()->clearMessage();

    printThread->wait();

    disconnect(printThread, 0);

    delete printThread;

    delete printer;
}

void MainWindow::saveCopy() {
    if(tabWidget->currentIndex() != -1) {
        QString path = settings.value("mainWindow/path", QDir::homePath()).toString();
        QString filePath = QFileDialog::getSaveFileName(this, tr("Save copy"), path, tr("Portable document format (*.pdf)"));

        if(filePath != QString::Null()) {
            DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

            if(!documentWidget->documentController()->saveCopy(filePath)) {
                QMessageBox::warning(this, tr("Warning"), tr("Could not save copy of document."));
            }
        }
    }
}

void MainWindow::fullscreen() {
    if(fullscreenAction->isChecked()) {
        geometryBeforeFullscreen = this->saveGeometry();
        this->showFullScreen();
    } else {
        this->restoreGeometry(geometryBeforeFullscreen);
        this->showNormal();
        this->restoreGeometry(geometryBeforeFullscreen);
    }
}

void MainWindow::about() {
    QMessageBox::about(this, tr("About qpdfview"), tr("<p><b>qpdfview</b></p><p>qpdfview is a tabbed PDF viewer using the poppler library.</p><p>&copy; 2012 Adam Reichold</p>"));
}

void MainWindow::updateDocument(const QString &filePath) {
    if(tabWidget->currentIndex() != -1) {
        tabWidget->setTabText(tabWidget->currentIndex(), QFileInfo(filePath).baseName());
        tabWidget->setTabToolTip(tabWidget->currentIndex(), QFileInfo(filePath).baseName());
    }
}

void MainWindow::updateScaleMode(const DocumentController::ScaleModes &scaleMode) {
    if(tabWidget->currentIndex() != -1) {
        switch(scaleMode) {
        case DocumentController::ScaleFactorMode:
            scaleFactorModeAction->setChecked(true);
            break;
        case DocumentController::FitToPageMode:
            fitToPageModeAction->setChecked(true);
            break;
        case DocumentController::FitToPageWidthMode:
            fitToPageWidthModeAction->setChecked(true);
            break;
        }
    }
}

void MainWindow::updateDisplayMode(const DocumentController::DisplayModes &displayMode) {
    if(tabWidget->currentIndex() != -1) {
        switch(displayMode) {
        case DocumentController::PagingMode:
            pagingModeAction->setChecked(true);
            break;
        case DocumentController::ScrollingMode:
            scrollingModeAction->setChecked(true);
            break;
        case DocumentController::DoublePagingMode:
            doublePagingModeAction->setChecked(true);
            break;
        case DocumentController::DoubleScrollingMode:
            doubleScrollingModeAction->setChecked(true);
            break;
        }
    }
}

void MainWindow::updateRotationMode(const DocumentController::RotationModes &rotationMode) {
    if(tabWidget->currentIndex() != -1) {
        switch(rotationMode) {
        case DocumentController::DoNotRotateMode:
            doNotRotateAction->setChecked(true);
            break;
        case DocumentController::RotateBy90Mode:
            rotateBy90Action->setChecked(true);
            break;
        case DocumentController::RotateBy180Mode:
            rotateBy180Action->setChecked(true);
            break;
        case DocumentController::RotateBy270Mode:
            rotateBy270Action->setChecked(true);
            break;
        }
    }
}

// private slots

void MainWindow::updateCurrentTab(const int &index) {
    if(index != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        disconnect(previousPageAction, 0);
        disconnect(nextPageAction, 0);
        disconnect(firstPageAction, 0);
        disconnect(lastPageAction, 0);

        connect(previousPageAction, SIGNAL(triggered()), documentWidget->documentController(), SLOT(previousPage()));
        connect(nextPageAction, SIGNAL(triggered()), documentWidget->documentController(), SLOT(nextPage()));
        connect(firstPageAction, SIGNAL(triggered()), documentWidget->documentController(), SLOT(firstPage()));
        connect(lastPageAction, SIGNAL(triggered()), documentWidget->documentController(), SLOT(lastPage()));

        previousTabAction->setEnabled(true);
        nextTabAction->setEnabled(true);
        closeTabAction->setEnabled(true);

        reloadAction->setEnabled(true);
        printAction->setEnabled(true);
        saveCopyAction->setEnabled(true);

        previousPageAction->setEnabled(true);
        nextPageAction->setEnabled(true);
        firstPageAction->setEnabled(true);
        lastPageAction->setEnabled(true);

        scaleModeActionGroup->setEnabled(true);
        this->updateScaleMode(documentWidget->documentController()->scaleMode());

        displayModeActionGroup->setEnabled(true);
        this->updateDisplayMode(documentWidget->documentController()->displayMode());

        rotationModeActionGroup->setEnabled(true);
        this->updateRotationMode(documentWidget->documentController()->rotationMode());
    } else {
        previousTabAction->setEnabled(false);
        nextTabAction->setEnabled(false);
        closeTabAction->setEnabled(false);

        reloadAction->setEnabled(false);
        printAction->setEnabled(false);
        saveCopyAction->setEnabled(false);

        previousPageAction->setEnabled(false);
        nextPageAction->setEnabled(false);
        firstPageAction->setEnabled(false);
        lastPageAction->setEnabled(false);

        scaleFactorModeAction->setChecked(true);
        scaleModeActionGroup->setEnabled(false);

        pagingModeAction->setChecked(true);
        displayModeActionGroup->setEnabled(false);

        doNotRotateAction->setChecked(true);
        rotationModeActionGroup->setEnabled(false);
    }
}

void MainWindow::closeTabByIndex(const int &index) {
    DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->widget(index));

    if(tabWidget->currentIndex() == index) {
        disconnect(previousPageAction, 0);
        disconnect(nextPageAction, 0);
        disconnect(firstPageAction, 0);
        disconnect(lastPageAction, 0);
    }

    disconnect(documentWidget->documentController(), 0);

    tabWidget->removeTab(index);

    documentWidget->documentController()->storeSettings();

    delete documentWidget;
}

void MainWindow::setScaleModeByAction(QAction *scaleModeAction) {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        if(scaleModeAction==scaleFactorModeAction) {
            documentWidget->documentController()->setScaleMode(DocumentController::ScaleFactorMode);
        } else if(scaleModeAction==fitToPageModeAction) {
            documentWidget->documentController()->setScaleMode(DocumentController::FitToPageMode);
        } else if(scaleModeAction==fitToPageWidthModeAction) {
            documentWidget->documentController()->setScaleMode(DocumentController::FitToPageWidthMode);
        }
    }
}

void MainWindow::setDisplayModeByAction(QAction *displayModeAction) {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        if(displayModeAction == pagingModeAction) {
            documentWidget->documentController()->setDisplayMode(DocumentController::PagingMode);
        } else if(displayModeAction == scrollingModeAction) {
            documentWidget->documentController()->setDisplayMode(DocumentController::ScrollingMode);
        } else if(displayModeAction == doublePagingModeAction) {
            documentWidget->documentController()->setDisplayMode(DocumentController::DoublePagingMode);
        } else if(displayModeAction == doubleScrollingModeAction) {
            documentWidget->documentController()->setDisplayMode(DocumentController::DoubleScrollingMode);
        }
    }
}

void MainWindow::setRotationModeByAction(QAction *rotationModeAction) {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        if(rotationModeAction == doNotRotateAction) {
            documentWidget->documentController()->setRotationMode(DocumentController::DoNotRotateMode);
        } else if(rotationModeAction == rotateBy90Action) {
            documentWidget->documentController()->setRotationMode(DocumentController::RotateBy90Mode);
        } else if(rotationModeAction == rotateBy180Action) {
            documentWidget->documentController()->setRotationMode(DocumentController::RotateBy180Mode);
        } else if(rotationModeAction == rotateBy270Action) {
            documentWidget->documentController()->setRotationMode(DocumentController::RotateBy270Mode);
        }
    }
}

// events

void MainWindow::dragEnterEvent(QDragEnterEvent *dragEnterEvent) {
    if(dragEnterEvent->mimeData()->hasUrls()) {
        dragEnterEvent->acceptProposedAction();
    }
}

void MainWindow::dropEvent(QDropEvent *dropEvent) {
    if(dropEvent->mimeData()->hasUrls()) {
        dropEvent->acceptProposedAction();

        foreach(QUrl url, dropEvent->mimeData()->urls()) {
            if(url.scheme() == "file" && QFileInfo(url.path()).isFile()) {
                DocumentWidget *documentWidget = new DocumentWidget();

                if(documentWidget->documentController()->loadFile(url.path())) {
                    documentWidget->documentController()->loadSettings();

                    int index = tabWidget->addTab(documentWidget, QString::Null());

                    tabWidget->setTabText(tabWidget->currentIndex(), QFileInfo(url.path()).baseName());
                    tabWidget->setTabToolTip(tabWidget->currentIndex(), QFileInfo(url.path()).baseName());

                    tabWidget->setCurrentIndex(index);
                } else {
                    delete documentWidget;
                }
            }
        }
    }
}

void MainWindow::closeEvent(QCloseEvent *closeEvent) {
    if(tabWidget->currentIndex() != -1) {
        DocumentWidget *documentWidget = static_cast<DocumentWidget*>(tabWidget->currentWidget());

        documentWidget->documentController()->storeSettings();
    }

    if(fullscreenAction->isChecked()) {
        settings.setValue("mainWindow/geometry", geometryBeforeFullscreen);
    } else {
        settings.setValue("mainWindow/geometry", this->saveGeometry());
    }

    QMainWindow::closeEvent(closeEvent);
}
